# 概述
本项目旨在用 rust 开发一个基于 RISC-V 架构的操作系统内核（`S0S`），支持启动初始化、中断、I/O、进程管理、内存管理、执行文件解析、fat32 文件系统功能等，同时能够在真实硬件平台 K210 上运行。

本项目移植于 [rCore](https://github.com/rcore-os/rCore-Tutorial-v3)，使用了洛佳等开发的 [rustsbi](https://github.com/rustsbi/rustsbi)。使用 GPL v3 协议。

`Rust` 是一门同时追求安全、并发和性能的现代系统级编程语言，其丰富的类型系统和所有权模型保证了内存安全和线程安全，在编译期就能够消除各种各样的的错误，而其性能还能与 C/C++ 相媲美。得益于 rust 的诸多优秀特性，本项目使用 rust 进行开发。

`RISC-V` 是一个开源的精简指令集架构，相比于 x86 架构，其设计简单，且没有历史的包袱，因此我们更能专注于操作系统的设计。

`rCore` 是清华开发的一个开源操作系统，使用了模块化设计，虽然功能简单但五脏俱全，且拥有丰富的文档。我们的项目移植与此，添加使用了 fat32 文件系统，并更改和添加了许多系统调用，同时增加了多核功能（目前仅支持在 qemu 上模拟运行）。

# 环境
本项目使用的环境与 `rcore` 一样，可参考 `rcore` 相应的[环境配置](https://rcore-os.github.io/rCore-Tutorial-Book-v3/chapter0/5setup-devel-env.html)。[gdb插件配置](./环境.md)

# 启动
注：由于 `S0S` 需在评测机上进行评测，且该评测机不能联网，因此在编译的时候添加了 `--offline` 选项，而在本地启动运行的时候需要删除 `os/Makefile` 以及 `user/Makefile` 中 `cargo build` 后面的 `--offline` 选项，以便下载所需依赖。

另：`S0S` 启动时默认执行一系列系统调用测试后关机，因此若需要进入 `shell` 进行交互，需注释掉 `user/src/bin/user_shell.rs` 中的 `test()` 。

<center><img src="./doc/img/img1.jpg" style="zoom:80%;" /></center>

## qemu
进入 `os` 目录，执行 `make run` 即可在 `qemu` 上运行

## k210
先准备 `sdcard`，之后进入 `os` 目录执行 `make sdcard` 初始化 sd 卡，烧写文件系统镜像。

之后将 sd 卡插入 `k210`，并连接 `k210` 开发板，执行 `make run BOARD=k210` 烧写操作系统镜像，即可启动运行。

## 操作系统镜像

在项目根目录下执行 `make all` 可在根目录下获得操作系统镜像 `os.bin`

# 致谢

非常感谢刘国军老师的支持和指导以及陈天宇同学给予的帮助。