用目标三元组描述一个平台：CPU 架构、CPU 厂商、操作系统和运行时库（`riscv64gc-unknown-none-elf`）

# qemu 启动流程

在 Qemu 开始执行任何指令之前，首先两个文件将被加载到 Qemu 的物理内存中：即作为 bootloader 的 `rustsbi-qemu.bin` 被加载到物理内存以物理地址 `0x80000000` 开头的区域上，同时内核镜像 `os.bin` 被加载到以物理地址 `0x80200000` 开头的区域上。

- 将必要的文件载入到 Qemu 物理内存之后，Qemu CPU 的程序计数器会被初始化为 `0x1000` ，接下来它将执行寥寥数条指令并跳转到物理地址 `0x80000000` 对应的指令处并进入
- 在这一阶段，bootloader 负责对计算机进行一些初始化工作，并跳转到下一阶段软件的入口，在 Qemu 上即可实现将计算机控制权移交给我们的内核镜像 `os.bin` 。

# 内核地址空间

<center><img src="./img/img4.jpg" style="zoom:80%;" /></center>

# 应用地址空间

<center><img src="./img/img5.jpg" style="zoom:80%;" /></center>

# 进程资源释放

`exit` 系统调用，子进程地址空间中的逻辑段列表 `areas` 清空（即执行 `Vec` 向量清空），这将导致应用地址空间被回收（即进程的数据和代码对应的物理页帧都被回收）

`waitpid` 系统调用，父进程彻底回收子进程占用的所有资源：内核栈和它的 PID 还有它的应用地址空间存放页表的那些物理页帧等等。

# 线程资源释放

当一个线程执行完代表它的功能后，会通过 `exit` 系统调用退出。内核在收到线程发出的 `exit` 系统调用后，会回收线程占用的部分资源，即用户态用到的资源，比如用户态的栈，用于系统调用和异常处理的跳板页等。

而该线程的内核态用到的资源，比如内核栈等，需要通过进程/主线程调用 `waittid` 来回收了，这样整个线程才能被彻底销毁。

`exit` 系统调用，内核回收线程用户态资源

`waittid` 系统调用，主线程回收线程内核态资源

# 使用结构

栈式页帧管理策略

内核与用户地址空间分离

栈式进程标识符分配器

进程调度：时间片+队列

块缓存替换策略：队列

# 一些难点

## trap 切换

- `sstatus` 的 `SPP` 字段会被修改为 CPU 当前的特权级（U/S）。
- `sepc` 会被修改为 Trap 处理完成后默认会执行的下一条指令的地址（引起 trap 的指令地址）。
- `scause/stval` 分别会被修改成这次 Trap 的原因以及相关的附加信息。
- CPU 会跳转到 `stvec` 所设置的 Trap 处理入口地址，并将当前特权级设置为 S ，然后从Trap 处理入口地址处开始执行。

trap 切换时，比如从用户态切换到内核态，一开始（ch2）在 trap/mod.rs 的 `init` 中将 `stvec` 设置为 `__alltraps`，因此一旦 trap 之后，则跳转到 `__alltraps`进行处理

当回到用户态后，会从 `sepc` 取出执行的指令地址。于是在新建线程/进程时，把 `sepc` 设置为入口函数

## 为什么用户 trap 上下文不保存在内核栈中

要保存 trap 到内核栈中，需要内核地址空间的 token 写入到 `satp` 寄存器中，同时还要跳转到内核栈，因此需要更改 `sp`。而这就需要保存用户的这两个寄存器到临时寄存器中，但只有 `sscratch` 一个寄存器进行周转，所以不能将用户 trap 上下文保存到内核栈中

## `__alltraps` 中最后跳转使用寄存器 jr 而不是 call

在设置地址空间时，内核与应用的最高地址空间都被设置为 trampoline，而`.text.trampoline` 段在链接脚本中处于代码段，但实际被映射到最高地址空间中，也即 `.text.trampoline` 段在执行的虚拟地址与其它代码段是不连续的。使用 call 编译器会将跳转地址设置为 pc 自增一个偏移量，这明显不对。

## `trap_return` 为什么使用 `fence.i` 指令

 `trap_return` 最后跳转到 `__restore` 中进行 trap 上下文恢复，此时需要切换进程空间，因此后面的指令地址失效，所以需要清空指令缓存