use crate::CORE_NUM;

use super::{hartid, TaskControlBlock};
use alloc::collections::VecDeque;
use alloc::sync::Arc;
use alloc::vec::Vec;
use lazy_static::*;
use spin::Mutex;

pub struct TaskManager {
    ready_queue: VecDeque<Arc<TaskControlBlock>>,
}

/// A simple FIFO scheduler.
impl TaskManager {
    pub fn new() -> Self {
        Self {
            ready_queue: VecDeque::new(),
        }
    }
    pub fn add(&mut self, task: Arc<TaskControlBlock>) {
        self.ready_queue.push_back(task);
    }
    pub fn fetch(&mut self) -> Option<Arc<TaskControlBlock>> {
        self.ready_queue.pop_front()
    }
    pub fn remove(&mut self, task: Arc<TaskControlBlock>) {
        if let Some((id, _)) = self
            .ready_queue
            .iter()
            .enumerate()
            .find(|(_, t)| Arc::as_ptr(t) == Arc::as_ptr(&task))
        {
            self.ready_queue.remove(id);
        }
    }
}

lazy_static! {
    pub static ref TASK_MANAGERS: Vec<Mutex<TaskManager>> = {
        let mut res = Vec::with_capacity(CORE_NUM);
        for _ in 0..CORE_NUM {
            res.push(Mutex::new(TaskManager::new()));
        }
        res
    };
}

pub fn add_task(task: Arc<TaskControlBlock>) {
    TASK_MANAGERS[0].lock().add(task);
}

pub fn fetch_task() -> Option<Arc<TaskControlBlock>> {
    TASK_MANAGERS[0].lock().fetch()
}

pub fn remove_task(task: Arc<TaskControlBlock>) {
    TASK_MANAGERS[0].lock().remove(task);
}
