use core::ops::Add;

use crate::config::CLOCK_FREQ;
use crate::sbi::set_timer;
use riscv::register::time;

const TICKS_PER_SEC: usize = 100;
const MSEC_PER_SEC: usize = 1000;
const USEC_PER_SEC: usize = 1000000;
const NSEC_PER_SEC: usize = 1000000000;

pub struct Timeval {
    pub tv_sec: u64,
    pub tv_usec: u64,
}

#[repr(C)]
#[derive(Clone, Copy, Debug, PartialEq, PartialOrd)]
pub struct Timespec {
    pub tv_sec: usize,
    pub tv_nsec: usize,
}

impl Add for Timespec {
    type Output = Timespec;

    fn add(self, other: Timespec) -> Timespec {
        Timespec {
            tv_sec: self.tv_sec + other.tv_sec,
            tv_nsec: self.tv_nsec + other.tv_nsec,
        }
    }
}

pub struct Tms {
    pub tms_utime: u64,  /* 用户使用的 CPU 时间 */
    pub tms_stime: u64,  /* 系统（内核）CPU 时间 */
    pub tms_cutime: u64, /* 已终止的子进程使用的用户 CPU 时间 */
    pub tms_cstime: u64, /* 已终止的子进程使用的系统 CPU 时间 */
}

pub fn get_time() -> usize {
    time::read()
}

pub fn get_time_s() -> usize {
    time::read() / CLOCK_FREQ
}

#[allow(unused)]
pub fn get_time_ms() -> usize {
    time::read() / (CLOCK_FREQ / MSEC_PER_SEC)
}

pub fn get_time_us() -> usize {
    time::read() / (CLOCK_FREQ / USEC_PER_SEC)
}

pub fn get_time_ns() -> usize {
    time::read() / CLOCK_FREQ * NSEC_PER_SEC
}

pub fn current_timespec() -> Timespec {
    Timespec {
        tv_sec: get_time_s(),
        tv_nsec: get_time_ns(),
    }
}

pub fn set_next_trigger() {
    set_timer(get_time() + CLOCK_FREQ / TICKS_PER_SEC);
}
